/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.so.components.se.flowable;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.List;
import java.util.Properties;

import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.so.DefectEchoer;
import org.ow2.petals.cli.extension.command.monitoring.so.NagiosMapper;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.SubscriptionObjectBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.SubscriptionObjectException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.SubscriptionObjectMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.SubscriptionObjectMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.so.components.AbstractTest;
import org.ow2.petals.cli.extension.command.monitoring.so.components.framework.AbstractMonitoredComponent;
import org.ow2.petals.cli.extension.command.monitoring.so.components.framework.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.api.mock.monitoring.DefaultDefectCreator;
import org.ow2.petals.jmx.api.mock.monitoring.DefaultDefectEmitterMock;
import org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock;

/**
 * Unit tests of {@link SeFlowable}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class SeFlowableTest extends AbstractTest {

    @Override
    protected AbstractMonitoredComponent buildComponent() {
        return new SeFlowable();
    }

    /**
     * Test the method 'subscribe()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test(expected = SubscriptionObjectMissingOptionsException.class)
    public void subscribe_noOptionProvided() throws SubscriptionObjectMissingArgumentException,
            SubscriptionObjectBadArgumentNumberException, SubscriptionObjectException {

        try {
            this.component.subscribe(new String[] {});
        } catch (final SubscriptionObjectMissingOptionsException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(SubscriptionObjectMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
            throw e;
        }
    }

    /**
     * Test the method 'subscribe()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test(expected = SubscriptionObjectBadArgumentNumberException.class)
    public void subscribe_unrecognizedOption() throws SubscriptionObjectMissingArgumentException,
            SubscriptionObjectBadArgumentNumberException, SubscriptionObjectException {

        try {
            this.component.subscribe(new String[] { "-z" });
        } catch (final SubscriptionObjectBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    SubscriptionObjectBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
            throw e;
        }
    }

    /**
     * Test the method 'subscribe()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the option associated to the SE Flowable identifier is provided without the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the missing argument of the option
     * </p>
     */
    @Test(expected = SubscriptionObjectMissingArgumentException.class)
    public void subscribe_componentIdMissing() throws SubscriptionObjectException {

        try {
            this.component.subscribe(new String[] { "-" + SeFlowable.COMPONENT_NAME_SHORT_OPTION });
        } catch (final SubscriptionObjectMissingArgumentException e) {
            assertEquals("Unexpected subscription object", this.component.getName(),
                    e.getSubscriptionObject().getName());
            assertEquals("Unexpected option", SeFlowable.COMPONENT_NAME_SHORT_OPTION, e.getOption().getOpt());
            throw e;
        }
    }

    /**
     * Test the method 'subscribe()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the option associated to the SE Flowable identifier is set to a valid value,</li>
     * <li>the option associated to the Nagios mapping file is set without value.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the missing argument of the option
     * </p>
     */
    @Test(expected = SubscriptionObjectMissingArgumentException.class)
    public void subscribe_nagiosMappingFileMissing() throws SubscriptionObjectException {

        try {
            this.component.subscribe(new String[] { "-" + SeFlowable.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "-" + SeFlowable.MAPPING_FILE_URL_SHORT_OPTION });
        } catch (final SubscriptionObjectMissingArgumentException e) {
            assertEquals("Unexpected subscription object", this.component.getName(),
                    e.getSubscriptionObject().getName());
            assertEquals("Unexpected option", SeFlowable.MAPPING_FILE_URL_SHORT_OPTION, e.getOption().getOpt());
            throw e;
        }
    }

    /**
     * Test the method 'subscribe()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the option associated to the SE Flowable identifier is set to a valid value,</li>
     * <li>the option associated to the Nagios mapping file is set to an inexisting file.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the inexisting mapping rule file
     * </p>
     */
    @Test(expected = SubscriptionObjectException.class)
    public void subscribe_inexistingNagiosMappingFile() throws SubscriptionObjectException {

        final String inexistingFile = "inexisting-file";
        try {
            this.component.subscribe(new String[] { "-" + SeFlowable.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "-" + SeFlowable.MAPPING_FILE_URL_SHORT_OPTION, inexistingFile });
        } catch (final SubscriptionObjectException e) {
            assertEquals("Unexpected subscription object", this.component.getName(),
                    e.getSubscriptionObject().getName());
            assertTrue("Unexpected inexisting file in message", e.getMessage().contains(inexistingFile));
            throw e;
        }
    }

    /**
     * Test the method 'subscribe()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided does not exist.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test(expected = ComponentDoesNotExistException.class)
    public void subscribe_componentIdDoesNotExist() throws SubscriptionObjectException {

        final String expecteComponentIdentifier = SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT;

        try {
            this.component.subscribe(
                    new String[] { "-" + SeFlowable.COMPONENT_NAME_SHORT_OPTION, expecteComponentIdentifier });
        } catch (final ComponentDoesNotExistException e) {
            assertEquals("Unexpected BC SOAP identifier", expecteComponentIdentifier, e.getComponentIdentifier());
            assertEquals("Unexpected subscription object", this.component.getName(),
                    e.getSubscriptionObject().getName());
            throw e;
        }
    }

    /**
     * Test the method 'subscribe()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>error occurs subscribing to the JMX notification.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = SubscriptionObjectException.class)
    public void subscribe_errorOnSubscription() throws SubscriptionObjectException {

        try {
            this.component.subscribe(new String[] { "-" + SeFlowable.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR });
        } catch (final SubscriptionObjectException e) {
            assertEquals("Unexpected subscription object", this.component.getName(),
                    e.getSubscriptionObject().getName());
            throw e;
        }
    }

    /**
     * Test the method 'subscribe()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the option associated to the SE Flowable identifier is set to a valid value,</li>
     * <li>the option associated to the Nagios mapping file is set to an valid file.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>mapping are correctly applied.</li>
     * </ul>
     * </p>
     */
    @Test
    public void subscribe_invalidNagiosMappingFile()
            throws SubscriptionObjectException, IOException, InterruptedException {

        // Expected Nagios host name and service name
        final String expectedNagiosHost = "nagios-host";
        final String expectedNagiosSvc = "nagios-svc";

        // Generate the mapping file
        final Properties mappingRules = new Properties();
        mappingRules.put(NagiosMapper.EMITTER_PREFIX + DefaultDefectCreator.DEFECT_EMITTER_NAME, expectedNagiosHost);
        mappingRules.put(NagiosMapper.NAME_PREFIX + DefaultDefectCreator.DEFECT_TYPE, expectedNagiosSvc);
        final File mappingRulesFile = this.temporaryFolder.newFile("mappingrules.properties");
        final OutputStream fos = new FileOutputStream(mappingRulesFile);
        try {
            mappingRules.store(fos, null);
        } finally {
            fos.close();
        }

        this.component.subscribe(new String[] { "-" + SeFlowable.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + SeFlowable.MAPPING_FILE_URL_SHORT_OPTION, mappingRulesFile.toURI().toURL().toString() });

        // Wait defect notification
        Thread.sleep((long) (DefaultDefectEmitterMock.PERIOD * 1.5));
        this.component.unsubscribe(new String[] { "-" + SeFlowable.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });

        final String error = systemErr.getLog();
        final String output = systemOut.getLog();

        assertFalse("No defect received", output.length() == 0);
        assertTrue("An error message was displayed", error.length() == 0);

        final StringBuilder helpMsgRegexp = new StringBuilder();
        helpMsgRegexp.append("^\\n*\\[[0-9]*\\] ");
        helpMsgRegexp.append(DefectEchoer.NAGIOS_COMMAND);
        helpMsgRegexp.append(';');
        helpMsgRegexp.append(expectedNagiosHost);
        helpMsgRegexp.append(';');
        helpMsgRegexp.append(expectedNagiosSvc);
        helpMsgRegexp.append(';');
        helpMsgRegexp.append(DefectEchoer.NAGIOS_CRITICAL);
        helpMsgRegexp.append(';');
        helpMsgRegexp.append(".*\\n$");
        assertTrue("Invalid Nagios command or mapping fails: " + output, output.matches(helpMsgRegexp.toString()));
    }

    /**
     * Test the method 'unsubscribe()' against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test(expected = SubscriptionObjectMissingOptionsException.class)
    public void unsubscribe_noOptionProvided() throws SubscriptionObjectMissingArgumentException,
            SubscriptionObjectBadArgumentNumberException, SubscriptionObjectException {

        try {
            this.component.unsubscribe(new String[] {});
        } catch (final SubscriptionObjectMissingOptionsException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(SubscriptionObjectMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
            throw e;
        }
    }

    /**
     * Test the method 'unsubscribe()' against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test(expected = SubscriptionObjectBadArgumentNumberException.class)
    public void unsubscribe_unrecognizedOption() throws SubscriptionObjectMissingArgumentException,
            SubscriptionObjectBadArgumentNumberException, SubscriptionObjectException {

        try {
            this.component.unsubscribe(new String[] { "-z" });
        } catch (final SubscriptionObjectBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    SubscriptionObjectBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
            throw e;
        }
    }

    /**
     * Test the method 'unsubscribe()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the option associated to the SE Flowable identifier is provided without the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the missing argument of the option
     * </p>
     */
    @Test(expected = SubscriptionObjectMissingArgumentException.class)
    public void unsubscribe_componentIdMissing() throws SubscriptionObjectException {

        try {
            this.component.unsubscribe(new String[] { "-" + SeFlowable.COMPONENT_NAME_SHORT_OPTION });
        } catch (final SubscriptionObjectMissingArgumentException e) {
            assertEquals("Unexpected subscription object", this.component.getName(),
                    e.getSubscriptionObject().getName());
            assertEquals("Unexpected option", SeFlowable.COMPONENT_NAME_SHORT_OPTION, e.getOption().getOpt());
            throw e;
        }
    }

    /**
     * Test the method 'unsubscribe()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the option associated to the SE Flowable identifier is set to a valid value,</li>
     * <li>the option associated to the Nagios mapping file is set without value.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the missing argument of the option
     * </p>
     */
    @Test(expected = SubscriptionObjectMissingArgumentException.class)
    public void unsubscribe_nagiosMappingFileMissing() throws SubscriptionObjectException {

        try {
            this.component.unsubscribe(new String[] { "-" + SeFlowable.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "-" + SeFlowable.MAPPING_FILE_URL_SHORT_OPTION });
        } catch (final SubscriptionObjectMissingArgumentException e) {
            assertEquals("Unexpected subscription object", this.component.getName(),
                    e.getSubscriptionObject().getName());
            assertEquals("Unexpected option", SeFlowable.MAPPING_FILE_URL_SHORT_OPTION, e.getOption().getOpt());
            throw e;
        }
    }

    /**
     * Test the method 'unsubscribe()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided does not exist.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test(expected = ComponentDoesNotExistException.class)
    public void unsubscribe_componentIdDoesNotExist() throws SubscriptionObjectException {

        final String expecteComponentIdentifier = SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT;

        try {
            this.component.unsubscribe(
                    new String[] { "-" + SeFlowable.COMPONENT_NAME_SHORT_OPTION, expecteComponentIdentifier });
        } catch (final ComponentDoesNotExistException e) {
            assertEquals("Unexpected BC SOAP identifier", expecteComponentIdentifier, e.getComponentIdentifier());
            assertEquals("Unexpected subscription object", this.component.getName(),
                    e.getSubscriptionObject().getName());
            throw e;
        }
    }

    /**
     * Test the method 'unsubscribe()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>error occurs unsubscribing to the JMX notification.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = SubscriptionObjectException.class)
    public void unsubscribe_errorOnUnsubcription() throws SubscriptionObjectException {

        try {
            this.component.unsubscribe(new String[] { "-" + SeFlowable.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR });
        } catch (final SubscriptionObjectException e) {
            assertEquals("Unexpected subscription object", this.component.getName(),
                    e.getSubscriptionObject().getName());
            throw e;
        }
    }

    /**
     * Test the method 'unsubscribe()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the option associated to the SE Flowable identifier is set to a valid value,</li>
     * <li>the option associated to the Nagios mapping file is set to an inexisting file.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test(expected = SubscriptionObjectBadArgumentNumberException.class)
    public void unsubscribe_inexistingNagiosMappingFile() throws SubscriptionObjectException {

        final String inexistingFile = "inexisting-file";
        try {
            this.component.unsubscribe(new String[] { "-" + SeFlowable.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "-" + SeFlowable.MAPPING_FILE_URL_SHORT_OPTION, inexistingFile });
        } catch (final SubscriptionObjectBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    SubscriptionObjectBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
            throw e;
        }
    }

    /**
     * Test the method 'unsubscribe()' against processing, where:
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the option associated to the SE Flowable identifier is set to a valid value,</li>
     * <li>the option associated to the Nagios mapping file is set to a valid file.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test(expected = SubscriptionObjectBadArgumentNumberException.class)
    public void unsubscribe_withValidArgs() throws SubscriptionObjectException, IOException {

        // Expected Nagios host name and service name
        final String expectedNagiosHost = "nagios-host";
        final String expectedNagiosSvc = "nagios-svc";

        // Generate the mapping file
        final Properties mappingRules = new Properties();
        mappingRules.put(NagiosMapper.EMITTER_PREFIX + DefaultDefectCreator.DEFECT_EMITTER_NAME, expectedNagiosHost);
        mappingRules.put(NagiosMapper.NAME_PREFIX + DefaultDefectCreator.DEFECT_TYPE, expectedNagiosSvc);
        final File mappingRulesFile = this.temporaryFolder.newFile("mappingrules.properties");
        final OutputStream fos = new FileOutputStream(mappingRulesFile);
        try {
            mappingRules.store(fos, null);
        } finally {
            fos.close();
        }

        final String inexistingFile = "inexisting-file";
        try {
            this.component.unsubscribe(new String[] { "-" + SeFlowable.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "-" + SeFlowable.MAPPING_FILE_URL_SHORT_OPTION, inexistingFile });
        } catch (final SubscriptionObjectBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    SubscriptionObjectBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
            throw e;
        }
    }
}
